#![recursion_limit = "256"]

use request::{ CustomParamRequest, CustomRequest, DoFastApi };

use digital_lsp::server::GLOBAL_FAST_MAP;
use log::info;
use std::{collections::HashMap, sync::{Arc, RwLock}};
use structopt::StructOpt;
use tower_lsp::{LspService, Server};

mod core;
mod completion;
mod definition;
mod hover;
mod utils;
mod diagnostics;
mod format;
mod server;
mod sources;
mod request;

use server::{Backend, GLOBAL_BACKEND};

#[derive(StructOpt, Debug)]
#[structopt(name = "Digtal LSP", about = "LSP for Digital IDE")]
struct Opt {}

#[tokio::main]
async fn main() {
    let _ = Opt::from_args();
    let log_handle = flexi_logger::Logger::with(flexi_logger::LogSpecification::info())
        .start()
        .unwrap();
    
    info!("launch Digital LSP");

    let stdin = tokio::io::stdin();
    let stdout = tokio::io::stdout();

    let (service, socket) = LspService::build(|client| {
        let backend = Arc::new(Backend::new(client, log_handle));

        // 初始化全局变量
        let _ = GLOBAL_BACKEND.set(backend.clone());
        let _ = GLOBAL_FAST_MAP.set(RwLock::new(HashMap::new()));

        backend
    })
        .custom_method("custom/request", CustomRequest)
        .custom_method("custom/paramRequest", CustomParamRequest)
        .custom_method("api/fast", DoFastApi)
        .finish();

    Server::new(stdin, stdout, socket)
        .serve(service)
        .await;
}
